#include "fw/dynamixel/base.h"

#include <schpin_lib_exp/protocol.h>

#pragma once

namespace schpin
{

enum dynmx_message_enum : uint8_t
{
        DYNMX_PING          = 0x01,
        DYNMX_READ          = 0x02,
        DYNMX_WRITE         = 0x03,
        DYNMX_REG_WRITE     = 0x04,
        DYNMX_ACTION        = 0x05,
        DYNMX_FACTORY_RESET = 0x06,
        DYNMX_REBOOT        = 0x08,
        DYNMX_CLEAR         = 0x10,
        DYNMX_STATUS        = 0x55,
        DYNMX_SYNC_READ     = 0x82,
        DYNMX_SYNC_WRITE    = 0x83,
        DYNMX_BULK_READ     = 0x92,
        DYNMX_BULK_WRITE    = 0x93
};

struct master_dynmx_group : command_group<>::with_msg_counter< uint16_t >::with_commands<
                                command< DYNMX_PING >,
                                command< DYNMX_READ >::with_args< dynmx_addr, dynmx_len >,
                                command< DYNMX_WRITE >::with_args< dynmx_addr, dynmx_buffer >,
                                command< DYNMX_REG_WRITE >::with_args< dynmx_addr, dynmx_buffer >,
                                command< DYNMX_ACTION >,
                                command< DYNMX_FACTORY_RESET >::with_args< dynmx_reset_flags >,
                                command< DYNMX_REBOOT >
                                // CLEAR
                                // DYNMX_SYNC_READ
                                // SYNC_WRITE
                                // BULK_READ
                                // BULK_WRITE
                                >
{
};

using dynmx_status_const = bounded< uint8_t, DYNMX_STATUS, DYNMX_STATUS >;
static constexpr dynmx_status_const dynmx_status_const_v =
    dynmx_status_const::get< DYNMX_STATUS >();

using master_dynmx_submessage = master_dynmx_group::message_t;
using master_dynmx_variant    = master_dynmx_group::command_variant;

using dynmx_master_protocol =
    protocol_tuple<>::with_msg_counter< uint16_t >::with_items< dynmx_header,
                                                                dynmx_servo_id,
                                                                dynmx_status_const,
                                                                dynmx_error,
                                                                dynmx_buffer,
                                                                dynmx_crc >;

using dynmx_master_tuple = dynmx_master_protocol::value_tuple;

using master_dynmx_protocol = protocol_tuple<>::
    with_items< dynmx_header, dynmx_servo_id, master_dynmx_submessage, dynmx_crc >;
using master_dynmx_message = master_dynmx_protocol::message_t;

template < dynmx_message_enum ID, typename... Args >
inline master_dynmx_message make_master_dynmx_msg( dynmx_servo_id id, Args... args )
{
        dynmx_crc crc;
        return master_dynmx_protocol::serialize(
            dynmx_header_v, id, master_dynmx_group::create< ID >( args... ), crc );
}

}  // namespace schpin
