#include "fw/base.h"

#include <schpin_lib_exp/protocol/config.h>

#pragma once

namespace schpin
{

enum servo_config_enum : uint8_t
{
        SC_INPUT_RANGE_LOW,
        SC_INPUT_RANGE_HIGH,
        SC_SERVO_RANGE_LOW,
        SC_SERVO_RANGE_HIGH,
        SC_ANGLE_CHANGE_REPORT,
        SC_ANGLE_STALL_RANGE,
        SC_GOTO_P_GAIN,
        SC_GOTO_I_GAIN,
        SC_GOTO_D_GAIN,
        SC_VEL_P_GAIN,
        SC_VEL_I_GAIN,
        SC_VEL_D_GAIN,
};

// multiple binding definitions.... 3x here and one should be a in lua_vm/def.h, this is not good
struct servo_config
{
        bangle      input_low;
        bangle      input_high;
        lewan_angle servo_low;
        lewan_angle servo_high;
        lewan_angle angle_change_report = lewan_angle::get< 5 >();
        lewan_angle angle_stall_range   = lewan_angle::get< 25 >();
        pid_gain    goto_p_gain         = pid_gain::from_float( 1.f );
        pid_gain    goto_i_gain         = pid_gain::from_float( 0.f );
        pid_gain    goto_d_gain         = pid_gain::from_float( 0.005f );
        pid_gain    vel_p_gain          = pid_gain::from_float( 0.f );
        pid_gain    vel_i_gain          = pid_gain::from_float( 0.005f );
        pid_gain    vel_d_gain          = pid_gain::from_float( 0.005f );

        auto bind_prot_config()
        {
                return std::make_tuple(
                    prot_conf_parameter< SC_INPUT_RANGE_LOW >( &input_low ),
                    prot_conf_parameter< SC_INPUT_RANGE_HIGH >( &input_high ),
                    prot_conf_parameter< SC_SERVO_RANGE_LOW >( &servo_low ),
                    prot_conf_parameter< SC_SERVO_RANGE_HIGH >( &servo_high ),
                    prot_conf_parameter< SC_ANGLE_CHANGE_REPORT >( &angle_change_report ),
                    prot_conf_parameter< SC_ANGLE_STALL_RANGE >( &angle_stall_range ),
                    prot_conf_parameter< SC_GOTO_P_GAIN >( &goto_p_gain ),
                    prot_conf_parameter< SC_GOTO_I_GAIN >( &goto_i_gain ),
                    prot_conf_parameter< SC_GOTO_D_GAIN >( &goto_d_gain ),
                    prot_conf_parameter< SC_VEL_P_GAIN >( &vel_p_gain ),
                    prot_conf_parameter< SC_VEL_I_GAIN >( &vel_i_gain ),
                    prot_conf_parameter< SC_VEL_D_GAIN >( &vel_d_gain ) );
        }
};

using servo_master_config_message =
    typename prot_config_types< servo_config >::config_master_message;
using servo_master_config_variant =
    typename prot_config_types< servo_config >::config_master_command_variant;
using master_servo_config_message =
    typename prot_config_types< servo_config >::master_config_message;
using master_servo_config_variant =
    typename prot_config_types< servo_config >::master_config_command_variant;

#ifdef SCHPIN_USE_NLOHMANN_JSON
inline void from_json( const nlohmann::json& j, servo_config& conf )
{
        // TODO: this is not good wat to change the config, have to figure out something better,
        // this is big fail
        if ( j.contains( "input_low" ) )
                conf.input_low = j["input_low"].get< bangle >();
        if ( j.contains( "input_high" ) )
                conf.input_high = j["input_high"].get< bangle >();
        if ( j.contains( "servo_low" ) )
                conf.servo_low = j["servo_low"].get< lewan_angle >();
        if ( j.contains( "servo_high" ) )
                conf.servo_high = j["servo_high"].get< lewan_angle >();
        if ( j.contains( "angle_change_report" ) )
                conf.angle_change_report = j["angle_change_report"].get< lewan_angle >();
        if ( j.contains( "angle_stall_range" ) )
                conf.angle_stall_range = j["angle_stall_range"].get< lewan_angle >();
        if ( j.contains( "goto_p_gain" ) )
                conf.goto_p_gain = j["goto_p_gain"].get< pid_gain >();
        if ( j.contains( "goto_i_gain" ) )
                conf.goto_i_gain = j["goto_i_gain"].get< pid_gain >();
        if ( j.contains( "goto_d_gain" ) )
                conf.goto_d_gain = j["goto_d_gain"].get< pid_gain >();
        if ( j.contains( "vel_p_gain" ) )
                conf.vel_p_gain = j["vel_p_gain"].get< pid_gain >();
        if ( j.contains( "vel_i_gain" ) )
                conf.vel_i_gain = j["vel_i_gain"].get< pid_gain >();
        if ( j.contains( "vel_d_gain" ) )
                conf.vel_d_gain = j["vel_d_gain"].get< pid_gain >();
}

inline void to_json( nlohmann::json& j, const servo_config& conf )
{
        j["input_low"]           = conf.input_low;
        j["input_high"]          = conf.input_high;
        j["servo_low"]           = conf.servo_low;
        j["servo_high"]          = conf.servo_high;
        j["angle_change_report"] = conf.angle_change_report;
        j["angle_stall_range"]   = conf.angle_stall_range;
        j["goto_p_gain"]         = conf.goto_p_gain;
        j["goto_i_gain"]         = conf.goto_i_gain;
        j["goto_d_gain"]         = conf.goto_d_gain;
        j["vel_p_gain"]          = conf.vel_p_gain;
        j["vel_i_gain"]          = conf.vel_i_gain;
        j["vel_d_gain"]          = conf.vel_d_gain;
}

#endif

}  // namespace schpin
