#include "fw/base.h"

#include <emlabcpp/physical_quantity.h>
#include <schpin_lib_exp/config.h>

#pragma once

namespace schpin
{

struct node_servo_config
{
        servo_id    id;
        std::string joint_name;
        lewan_angle from_servo;
        lewan_angle to_servo;
        em::angle   from_angle;
        em::angle   to_angle;

        static auto ros_param_get()
        {
                return std::make_tuple(  //
                    ros_param< servo_id >( "id", "id of the servo on lewansoul bus" ),
                    ros_param< std::string >( "joint_name",
                                              "name of the joint that is used for this servo" ),
                    ros_param< lewan_angle >( "from_servo",
                                              "lewan angle at which movement starts" ),
                    ros_param< lewan_angle >( "to_servo", "lewan angle at which movement ends" ),
                    ros_param< em::angle >( "from_angle", "angle at which movement starts" ),
                    ros_param< em::angle >( "to_angle", "angle at which movement ends" ) );
        }
};

struct node_hardware_config
{
        std::map< std::string, node_servo_config > servos;
        std::string                                diag_topic;
        float                                      diag_freq;
        std::string                                joint_state_topic;
        std::string                                robot_description_param;
        std::string                                robot_semantics_param;

        static auto ros_param_get()
        {
                return std::make_tuple(  //
                    ros_param< std::map< std::string, node_servo_config > >( "servos",
                                                                             "definitions of "
                                                                             "servomotors" ),
                    ros_param< std::string >( "diag_topic", "Diagnostics topic" ),
                    ros_param< float >( "diag_freq", "Diagnostic frequency" ),
                    ros_param< std::string >( "joint_state",
                                              "TOpic at which send updates of the joint "
                                              "positions." ),
                    ros_param< std::string >( "urdf", "Param at which urdf resides" ),
                    ros_param< std::string >( "srdf", "Param at which srdf resides" ) );
        }
};

}  // namespace schpin
