#include "shared/protocol.h"

#include <schpin_lib/lua/ser.h>

#pragma once

namespace schpin
{
template <>
struct lua_serializer< lewan_message_enum > : lua_enum_serializer< lewan_message_enum >
{
};
template <>
struct lua_serializer< messages_enum > : lua_enum_serializer< messages_enum >
{
};
template <>
struct lua_serializer< activated_enum > : lua_enum_serializer< activated_enum >
{
};
template <>
struct lua_serializer< prot_config_error_enum > : lua_enum_serializer< prot_config_error_enum >
{
};
template <>
struct lua_serializer< servo_task_enum > : lua_enum_serializer< servo_task_enum >
{
};

template < auto val >
struct lua_serializer< cmd_tag< val > >
{
        static void encode( lua_State* lua_ptr, cmd_tag< val > )
        {
                lua_enum_serializer< decltype( val ) >::encode( lua_ptr, val );
        }

        static std::string desc_name()
        {
                return std::string{ magic_enum::enum_name( val ) };
        }

        static std::string desc_example()
        {
                return std::string{ magic_enum::enum_entries< decltype( val ) >().front().second };
        }
};

template < typename MsgCounterType, std::size_t N, typename Owner >
struct lua_serializer< prot_message< MsgCounterType, N, Owner > >
{
        static void encode( lua_State* lua_ptr, const prot_message< MsgCounterType, N, Owner > msg )
        {
                std::stringstream ss;
                ss << msg;
                lua_serializer< std::string >::encode( lua_ptr, ss.str() );
        }

        static std::string desc_name()
        {
                return "Message of size " + to_string( N );
        }
};

template <>
struct lua_serializer< argument_index >
{
        static void encode( lua_State* lua_ptr, argument_index i )
        {
                lua_serializer< typename argument_index::value_type >::encode( lua_ptr, *i );
        }

        static std::string desc_name()
        {
                return lua_serializer< typename argument_index::value_type >::desc_name();
        }
};

}  // namespace schpin
