#include "schpin_lib/lua/def.h"

#include <schpin_lib/geom/pose.h>
#include <schpin_lib_exp/overload.h>

#pragma once

namespace schpin
{
using slua_pose_input_var =
    std::variant< point< 3, world_frame >, quaternion< world_frame >, pose< world_frame > >;

inline pose< world_frame > slua_get_real_pose( slua_pose_input_var var,
                                               pose< world_frame > actual_pose )
{
        var << match(
            [&]( point< 3, world_frame > p ) {
                    actual_pose.position = p;
            },
            [&]( quaternion< world_frame > q ) {
                    actual_pose.orientation = q;
            },
            [&]( pose< world_frame > p ) {
                    actual_pose = p;
            } );
        return actual_pose;
}

struct slua_pose_args
{
        using value_type = slua_pose_input_var;
        std::string point_key;
        std::string quat_key;
        std::string description;

        slua_pose_args( std::string pkey, std::string qkey, std::string desc )
          : point_key( pkey )
          , quat_key( qkey )
          , description( desc + " (you either specifiy both parts, or one of them. The missing one "
                                "is taken from actual pose.)" )
          , point_arg{ pkey, "" }
          , quat_arg{ qkey, "" }
        {
        }

        em::either< value_type, slua_error > table_extract( lua_State* lua_ptr,
                                                            int        tbl_index ) const
        {
                auto peither = point_arg.table_extract( lua_ptr, tbl_index );
                auto qeither = quat_arg.table_extract( lua_ptr, tbl_index );

                if ( peither.is_left() && !qeither.is_left() ) {
                        return peither.convert_left( [&]( auto p ) {  //
                                return slua_pose_input_var{ p };
                        } );
                }

                if ( qeither.is_left() && !peither.is_left() ) {
                        return qeither.convert_left( [&]( auto p ) {  //
                                return slua_pose_input_var{ p };
                        } );
                }

                return assemble_left_collect_right( peither, qeither )
                    .convert_left(
                        em::uncurry |
                        [&]( auto point, auto quat ) {
                                return slua_pose_input_var{ pose< world_frame >{ point, quat } };
                        } )
                    .convert_right( [&]( auto errs ) {
                            return E( slua_error ).group_attach( errs ) << "Failed to parse goal "
                                                                           "pose";
                    } );
        }

      private:
        lua_key_arg< point< 3, world_frame > >   point_arg;
        lua_key_arg< quaternion< world_frame > > quat_arg;
};

inline void to_json( nlohmann::json& j, const slua_pose_args& item )
{
        j["keys"] = { item.point_key, item.quat_key };
        j["desc"] = item.description;
        j["type"] = lua_serializer< point< 3, world_frame > >::desc_name() + ", " +
                    lua_serializer< quaternion< world_frame > >::desc_name();
        j["example"] = item.point_key + "=" +
                       lua_serializer< point< 3, world_frame > >::desc_example() + ", " +
                       item.quat_key + "=" +
                       lua_serializer< quaternion< world_frame > >::desc_example();
}

}  // namespace schpin
