#include <schpin_lib_exp/protocol/base.h>

#pragma once

namespace schpin
{

template < auto ID, typename T >
struct protocol_reg
{
        static constexpr auto id = ID;
        using value_type         = T;

        using prot_item                   = protocol_item< value_type >;
        static constexpr std::size_t size = prot_item::size;

        T value;
};

template < typename... >
struct protocol_reg_storage;

template <>
struct protocol_reg_storage<>
{

        template < auto ID >
        static constexpr bool contains_id = false;

        template < auto ID >
        using value_t = void;
};

template < auto ID >
struct protocol_reg_storage_tag
{
};

template < typename Reg, typename... Regs >
struct protocol_reg_storage< Reg, Regs... > : protocol_reg_storage< Regs... >
{
        using sub = protocol_reg_storage< Regs... >;

        Reg reg_;

        template < auto ID >
        static constexpr bool contains_id = ID == Reg::id ? true : sub::template contains_id< ID >;

        template < auto ID >
        using value_t = std::conditional_t< ID == Reg::id,
                                            typename Reg::value_type,
                                            typename sub::template value_t< ID > >;

        static constexpr std::size_t get_size( protocol_reg_storage_tag< Reg::id > )
        {
                return Reg::size;
        }

        typename Reg::value_type get_val( protocol_reg_storage_tag< Reg::id > )
        {
                return reg_.value;
        }

        void set_val( protocol_reg_storage_tag< Reg::id >, typename Reg::value_type val )
        {
                reg_.value = val;
        }
};

template < typename... Regs >
class protocol_map
{
        using storage = protocol_reg_storage< Regs... >;

        storage stor_;

        template < auto ID >
        using reg_value_t = typename storage::value_t< ID >;

        template < auto ID >
        static constexpr bool contains_id = storage::template contains_id< ID >;

        template < auto ID >
        static constexpr std::size_t get_size()
        {
                static_assert( contains_id< ID > );
                return storage::get_size( protocol_reg_storage_tag< ID >{} );
        }

        template < auto ID >
        reg_value_t< ID > get_val() const
        {
                static_assert( contains_id< ID > );
                return stor_.get_val( protocol_reg_storage_tag< ID >{} );
        }

        template < auto ID >
        void set_val( reg_value_t< ID > val )
        {
                static_assert( contains_id< ID > );
                stor_.set_val( protocol_reg_storage_tag< ID >{}, val );
        }
};

}  // namespace schpin
