#include "schpin_lib_exp/protocol/command_group.h"
#include "schpin_lib_exp/protocol/tuple.h"

#include <gtest/gtest.h>

using namespace schpin;

using test_prot =
    protocol_tuple<>::with_items< uint32_t, uint32_t, std::array< uint8_t, 4 >, std::bitset< 32 > >;
using test_msg = typename test_prot::message_t;

TEST( Protocol, tpl )
{
        std::array< uint8_t, 4 > data_arr{ 1, 2, 3, 4 };
        std::bitset< 32 >        bset{ 0xFF0055FF };
        test_msg                 msg      = test_prot::serialize( 0u, 55u, data_arr, bset );
        test_msg                 expected = test_msg{ std::array< uint8_t, 16 >{
            0, 0, 0, 0, 0, 0, 0, 55, 1, 2, 3, 4, 0xFF, 0x55, 0x00, 0xFF } };

        EXPECT_EQ( msg, expected );

        test_prot::extract( msg ).match(
            em::uncurry |
                [&](
                    uint32_t i, uint32_t j, std::array< uint8_t, 4 > vals, std::bitset< 32 > val ) {
                        EXPECT_EQ( i, 0 );
                        EXPECT_EQ( j, 55 );
                        EXPECT_EQ( vals, data_arr );
                        EXPECT_EQ( val, bset );
                },
            [&]( auto e ) {
                    FAIL() << e;
            } );
}

enum test_enum : uint8_t
{
        WOLOLO = 0,
        FOO    = 1
};

using test_cmd_prot = command_group<>::with_commands< command< WOLOLO >, command< FOO, uint16_t > >;
using test_cmd_msg  = typename test_cmd_prot::message_t;

TEST( Protocol, cmd_prot )
{
        auto wololo_msg = test_cmd_prot::create< WOLOLO >();

        test_cmd_prot::match(
            wololo_msg,
            [&]( cmd_tag< WOLOLO > ) {
                    SUCCEED();
            },
            [&]( cmd_tag< FOO >, uint16_t ) {
                    FAIL();
            },
            [&]( error_tag, auto... ) {
                    FAIL();
            } );

        auto foo_msg = test_cmd_prot::create< FOO >( uint16_t{ 22 } );

        test_cmd_prot::match(
            foo_msg,
            [&]( cmd_tag< WOLOLO > ) {
                    FAIL();
            },
            [&]( cmd_tag< FOO >, uint16_t val ) {
                    EXPECT_EQ( val, 22 );
            },
            [&]( error_tag, auto... ) {
                    FAIL();
            } );
}

using test_nested_prot = protocol_tuple<>::with_items< uint32_t, test_cmd_msg >;
using test_nested_msg  = typename test_nested_prot::message_t;

TEST( Protocol, nested )
{
        auto foo_msg = test_cmd_prot::create< FOO >( uint16_t{ 22 } );

        test_nested_msg msg = test_nested_prot::serialize( 0u, foo_msg );
        test_nested_msg expected{ std::array< uint8_t, 8 >{ 0, 0, 0, 0, 3, FOO, 0, 22 } };

        EXPECT_EQ( msg, expected );

        test_nested_prot::extract( msg ).match(
            em::uncurry |
                [&]( uint16_t i, test_cmd_msg submsg ) {
                        EXPECT_EQ( i, 0 );
                        EXPECT_EQ( submsg, foo_msg );
                },
            [&]( auto e ) {
                    FAIL() << e;
            }

        );
}
