#include "environment/octree.h"
#include "octree/load.h"
#include "octree/occupancy.h"
#include "octree/tree.h"
#include "schpin_lib/either.h"
#include "schpin_lib/error.h"

#include <filesystem>
#include <nlohmann/json.hpp>

#pragma once

namespace schpin
{

using env_proxy_node = oc_proxy_node< environment_oc_tree_content, world_frame >;
using env_oc_tree    = oc_tree< environment_oc_tree_content, world_frame >;

class environment
{
        env_oc_tree tree_;

      public:
        environment() = default;

        explicit environment( env_oc_tree&& tree )
          : tree_( std::move( tree ) )
        {
        }

        environment copy() const
        {
                return environment( tree_.copy() );
        }

        const env_oc_tree& get_tree() const
        {
                return tree_;
        }

        template < collision_mode Mode >
        oc_node_collision_query< Mode, environment_oc_tree_content, world_frame > get_query() const
        {
                return oc_node_collision_query< Mode, environment_oc_tree_content, world_frame >{
                    tree_.get_proxy_root()};
        }
};

inline std::ostream& operator<<( std::ostream& os, const environment& /*env*/ )
{
        return os;
}

using env_error = error< struct env_errorTag >;

struct env_config
{
        uint16_t                             default_occ;
        em::length                           min_edge_length;
        std::vector< std::filesystem::path > paths;
};

em::either< environment, env_error > build_environment( const env_config& config );

}  // namespace schpin

template <>
struct nlohmann::adl_serializer< schpin::environment >
{
        static void to_json( nlohmann::json& j, const schpin::environment& env )
        {
                j["tree"] = env.get_tree();
        }

        static schpin::environment from_json( const nlohmann::json& j )
        {
                return schpin::environment( j.at( "tree" ).get< schpin::env_oc_tree >() );
        }
};

// TODO: THIS IS BUG< SOLVE!
template <>
struct nlohmann::adl_serializer< const schpin::environment >
{
        static void to_json( nlohmann::json& j, const schpin::environment& env )
        {
                j["tree"] = env.get_tree();
        }

        static schpin::environment from_json( const nlohmann::json& j )
        {
                return schpin::environment( j.at( "tree" ).get< schpin::env_oc_tree >() );
        }
};

template <>
struct std::hash< schpin::environment >
{
        std::size_t operator()( const schpin::environment& env )
        {
                return std::hash< schpin::env_oc_tree >()( env.get_tree() );
        }
};
