#include "base/physics.h"
#include "schpin_lib/geom/point.h"
#include "schpin_lib/identificator.h"

#pragma once

namespace schpin
{

enum class oc_node_control
{
        FOLLOW_CHILDREN,
        END
};

enum class oc_locked_leaf_control
{
        EXPAND_CHILDREN,
        UNLOCK,
        END
};

using oc_child_index    = identificator< struct OcChildIndexTag, 8 >;
using oc_child_unit_pos = point< 3, struct OcChildFrame >;

template < typename Tag >
inline oc_child_index get_oc_child_index_based_on_center( const point< 3, Tag >& parent,
                                                          const point< 3, Tag >& child )
{
        std::bitset< 3 > bits;
        bits[0] = parent[0] > child[0] ? 0 : 1;
        bits[1] = parent[1] > child[1] ? 0 : 1;
        bits[2] = parent[2] > child[2] ? 0 : 1;
        return oc_child_index{ bits.to_ulong() };
}

inline oc_child_unit_pos get_oc_child_unit_pos( oc_child_index index )
{
        std::bitset< 3 > bits{ unsigned( *index ) };
        return oc_child_unit_pos{ bits[0] ? 1 : -1, bits[1] ? 1 : -1, bits[2] ? 1 : -1 };
}

template < typename Tag >
inline point< 3, Tag > get_oc_child_offset( oc_child_index index, em::length parent_edge )
{
        oc_child_unit_pos upos = get_oc_child_unit_pos( index );
        float             step = *parent_edge / 4.f;

        return point< 3, Tag >{ upos[0] * step, upos[1] * step, upos[2] * step };
}

}  // namespace schpin
