#include "octree/proxy_node.h"
#include "octree/tree.h"
#include "schpin_lib_exp/rviz.h"

#include <queue>
#include <ros/ros.h>

#pragma once

namespace schpin
{

inline visualization_msgs::Marker create_oc_marker( const std::string& name,
                                                    unsigned           id,
                                                    em::length         scale,
                                                    const std::string& frame,
                                                    float              alpha )
{
        visualization_msgs::Marker res = create_marker( name, id, frame );

        res.frame_locked = 1u;

        res.type = visualization_msgs::Marker::CUBE_LIST;

        res.action = visualization_msgs::Marker::ADD;

        res.scale.x = double( *scale );
        res.scale.y = double( *scale );
        res.scale.z = double( *scale );

        res.color.r = 0.5f;
        res.color.g = 0.5f;
        res.color.b = 0.5f;
        res.color.a = alpha;

        return res;
}

template < typename NodeContent, typename Tag, typename UnaryFunction >
visualization_msgs::MarkerArray create_oc_visualization( const std::string&                 name,
                                                         const oc_tree< NodeContent, Tag >& tree,
                                                         const std::string&                 frame,
                                                         float                              alpha,
                                                         UnaryFunction filter_f )
{
        const oc_proxy_node< NodeContent, Tag >         root = tree.get_proxy_root();
        std::queue< oc_proxy_node< NodeContent, Tag > > que;
        visualization_msgs::MarkerArray                 res;

        que.push( root );

        while ( !que.empty() ) {
                oc_proxy_node< NodeContent, Tag > node = que.front();
                que.pop();

                while ( res.markers.size() <= node.get_depth() ) {
                        res.markers.push_back( create_oc_marker(
                            name, node.get_depth(), node.get_edge_length(), frame, alpha ) );
                }
                if ( !node.has_children() && filter_f( *node ) ) {
                        geometry_msgs::Point coord = point_to_msg( node.get_position() );
                        res.markers[node.get_depth()].points.push_back( coord );
                }
                if ( node.has_children() ) {
                        for ( oc_proxy_node< NodeContent, Tag > child : node.generate_children() ) {
                                que.push( child );
                        }
                }
        }

        return res;
}

}  // namespace schpin
