#include "environment.h"

using namespace schpin;

int main( int argc, char* argv[] )
{
        ros::Time::init();
        if ( argc != 4 ) {
                SCHPIN_WARN_LOG( "env_to_util", "Bad number of arguments" );
        }

        std::string input_filename{argv[1]};
        double      edge_length = std::stod( argv[2] );
        std::string output_filename{argv[3]};

        env_config config;

        config.default_occ     = env_min_occupancy;
        config.min_edge_length = em::length( float( edge_length ) );
        config.paths.emplace_back( input_filename );

        build_environment( config ).match(
            [&]( environment&& env ) {
                    std::ofstream output{output_filename};
                    safe_store_json( output, env ).optionally( []( const json_storage_error& e ) {
                            SCHPIN_WARN_LOG( "env_to_util", e );
                    } );
            },
            [&]( const env_error& e ) { SCHPIN_WARN_LOG( "env_to_util", e ); } );

        return 0;
}
